local InventorySetWindow = require("UI/inventorySetWindow")
local PopupMenu = require("UI/InventoryWindowPopup")
local ThemeTool = require("UI/themeTool")

-- UI for home section of inventory setup --
local InventorySetupWindow = {
    onClose = nil,
    onRefresh = nil,
    onExport = nil,

    window = nil,
    panel = nil,

    compactMode = false,
    compactToggle = nil,

    sortingMode = false,
    sortingToggle = nil,

    sectionMode = false,
    sectionToggle = nil,
    sections = {},

    createButton = nil,
    importButton = nil,

    searchBar = nil,
    pastSearchValue = "",
    search = nil,

    sets = nil,
    themeDropdown = nil,
    closeButton = nil,

    initialize = function(self, sets, sections)
        local taskList = AsyncTaskList.New()  
        AsyncTaskList.schedule(taskList,function(callback) self:refresh(sets, sections) callback() end)  
        AsyncTaskList.schedule(taskList,function(callback) self:show() callback() end)
        AsyncTaskList.runTaskList(taskList)
    end,

    refresh = function(self, sets, sections)
        self.window = osrs.Ui:window(
            {
                title = "Inventory Setups",
                collapsable = false,
                sizeMode = 1,
                width = 300,
                height = 800,
                positionMode = 1,
            }
        )
        self.panel = osrs.Ui:pane(
            {
                border = false
            }
        )
        self.sortingToggle = osrs.Ui:imageButton(
            {
                sameLine = true,
                tooltip = "Sorting Mode",
                sizeMode = 1,
                width = 20,
                height = 20,
                onClick = function()
                    self.sortingMode = not self.sortingMode
                    self:clear()
                    self.onRefresh()
                    self:checkForSearch(true, sets, sections)
                end
            }
        )
        self.sortingToggle:setImagePath("Icons/sorting_icon.png")
        self:buttonColoring(self.sortingToggle, self.sortingMode)
        self.compactToggle = osrs.Ui:imageButton(
            {
                sameLine = true,
                tooltip = "Compact Mode",
                sizeMode = 1,
                width = 20,
                height = 20,
                onClick = function()
                    self.compactMode = not self.compactMode
                    self:clear()
                    self.onRefresh()
                    self:checkForSearch(true, sets, sections)
                end
            }
        )
        self.compactToggle:setImagePath("Icons/compact_icon.png")
        self:buttonColoring(self.compactToggle, self.compactMode)
        self.sectionToggle = osrs.Ui:imageButton(
            {
                sameLine = true,
                tooltip = "Section Mode",
                sizeMode = 1,
                width = 20,
                height = 20,
                onClick = function()
                    self.sectionMode = not self.sectionMode
                    self:clear()
                    self.onRefresh()
                    self:checkForSearch(true, sets, sections)
                end
            }
        )
        self.sectionToggle:setImagePath("Icons/section_icon.png")
        self:buttonColoring(self.sectionToggle, self.sectionMode)
        self.createButton = osrs.Ui:textButton(
            {
                tooltip = "Create",
                text = "+",
                onClick = function()
                    self.popupMenu:createPopup()
                end
            }
        )
        self.importButton = osrs.Ui:imageButton(
            {
                sameLine = true,
                tooltip = "Import",
                sizeMode = 1,
                width = 20,
                height = 20,
                onClick = function()
                    self.popupMenu:importPopup()
                end
            }
        )
        self.importButton:setImagePath("Icons/import_icon.png")
        self.searchBar = osrs.Ui:inputText(
            {
                label = "Search",
                sizeMode = 1,
                width = self.window.height / 4,
                height = 30,
                value = self.pastSearchValue
            }
        )
        if self.search == nil then
            self.search = osrs.Events:subscribe(function() self:checkForSearch(false, sets, sections) end, osrs.Events.ON_GAME_TICK)
        end
        self.themeDropdown = osrs.Ui:dropdown(
            {
                label = "Theme",
                selected = ThemeTool.currentTheme,
                changed = function ()
                    ThemeTool:setTheme(self.themeDropdown.selected)
                    self:clear()
                    self.onRefresh()
                    self:checkForSearch(true, sets, sections)
                end
            }
        )
        for i, theme in pairs(ThemeTool:getThemes()) do
            self.themeDropdown.values[i] = theme.name
        end
        self.closeButton = osrs.Ui:textButton(
            {
                text = "Close",
                onClick = function()
                    self.popupMenu:closeWindow()
                    self.onClose()
                end
            }
        )
        self:searchItems(sets, sections)
    end,

    buttonColoring = function(self, button, value)
        if value then 
            button.style.colour.button = osrs.Colour4.new(0.169, 0.671, 0.722, 1.0)
        end
    end,

    checkForSearch = function(self, run, sets, sections)
        if (self.searchBar.value ~= "" and self.searchBar.value ~= self.pastSearchValue) or run then 
            self.pastSearchValue = self.searchBar.value
            self:searchItems(sets, sections)
        elseif self.searchBar.value == "" and self.pastSearchValue ~= "" then
            self.pastSearchValue = ""
            self:searchItems(sets, sections)
        end
    end,

    searchSets = function(self, sets)
        local result = {}
        result["sets"] = {}
        result["sections"] = {}
        if #self.pastSearchValue > 5 and string.sub(self.pastSearchValue, 1, 5) == "item:" then 
            local itemName = string.lower(string.sub(self.pastSearchValue, 6, #self.pastSearchValue))
            for index, set in pairs(sets) do
                if set:hasItem(itemName) == 1 then
                    table.insert(result["sets"], set)
                    self:searchHelper(set, result)
                end
            end
        elseif #self.pastSearchValue > 6 and string.sub(self.pastSearchValue, 1, 6) == "notes:" then 
            local noteName = string.lower(string.sub(self.pastSearchValue, 7, #self.pastSearchValue))
            for index, set in pairs(sets) do
                if set:checkWords(noteName, set.notes) == 1 then
                    table.insert(result["sets"], set)
                    self:searchHelper(set, result)
                end
            end
        else
            for index, set in pairs(sets) do
                name = string.lower(string.sub(set.name, 1, #self.pastSearchValue))
                if name == string.lower(self.pastSearchValue) then 
                    table.insert(result["sets"], set)
                    self:searchHelper(set, result)
                end
            end
        end
        return result
    end,

    searchHelper = function(self, set, result)
        for _, section in pairs(set.sections) do
            local found = 0
            for _, saved in pairs(result["sections"]) do
                if section == saved then 
                    found = 1
                end
            end
            if found == 0 then 
                table.insert(result["sections"], section)
            end
        end
    end,

    setupSections = function(self, sections, list)
        for index, section in pairs(sections) do
            current = osrs.Ui:collapsingHeader(
                {
                    text = section,
                }
            )
            editButton = osrs.Ui:textButton(
                {
                    text = "Edit",
                    onClick = function()
                        self.popupMenu:holderPopup(section, index, #sections, "Section", nil, list, self.sectionMode)
                    end
                }
            )
            exportButton = osrs.Ui:imageButton(
                {
                    sameLine = true,
                    tooltip = "Export Section",
                    sizeMode = 1,
                    width = 20,
                    height = 20,
                    onClick = function()
                        self.onExport(section)
                    end
                }
            )
            exportButton:setImagePath("Icons/export_icon.png")
            current:addChild(editButton)
            current:addChild(exportButton)
            table.insert(self.sections, current)
        end
        Unassigned = osrs.Ui:collapsingHeader(
                {
                    text = "Unassigned",
                }
            )
        table.insert(self.sections, Unassigned)
    end,

    setupSets = function(self, sets)
        tempSets = {}
        if self.sortingMode then
            for index, set in pairs(sets) do
                tempSets[index] = set
            end
            table.sort(tempSets, function(a,b)
                return Inventory:alphabeticalCompare(a,b) end)
        else
            tempSets = sets
            for index, set in pairs(tempSets) do 
                if set.favorite then 
                    self:createSet(set, #tempSets)
                end
            end
        end
        for index, set in pairs(tempSets) do 
            if self.sortingMode or set.favorite == false then 
                self:createSet(set, #tempSets)
            end
        end
    end,

    createSet = function(self, set, total)
        local panel = self:clearPanelHelper()
        local setName = osrs.Ui:textbox({text = set.name})
        local editButton = self:editButtonHelper(nil, set, total)
        local viewButton = osrs.Ui:imageButton(
            {
                tooltip = "View Setup",
                sizeMode = 1,
                width = 20,
                height = 20,
                onClick = function()
                    self.popupMenu:closeWindow()
                    InventorySetWindow:initialize(set)
                    self:clear()
                end
            }
        )
        viewButton:setImagePath("Icons/view_icon.png")
        local orderToggle = osrs.Ui:imageButton(
            {
                sameLine = true,
                tooltip = "Only highlight items that are missing from the setup",

                sizeMode = 1,
                width = 20,
                height = 20,
                onClick = function()
                    set.ordered = not set.ordered
                    self:clear()
                    self.onRefresh()
                end
            }
        )
        orderToggle:setImagePath("Icons/order_icon.png")
        self:buttonColoring(orderToggle, set.ordered)
        local highlightToggle = osrs.Ui:imageButton(
            {
                sameLine = true,
                tooltip = "Enable highlighting",
                sizeMode = 1,
                width = 20,
                height = 20,
                onClick = function()
                    set.highlighting = not set.highlighting
                    self:clear()
                    self.onRefresh()
                end
            }
        )
        highlightToggle:setImagePath("Icons/highlight_icon.png")
        self:buttonColoring(highlightToggle, set.highlighting)
        local favToggle = osrs.Ui:imageButton(
            {
                sameLine = true,
                tooltip = "Favorite",
                sizeMode = 1,
                width = 20,
                height = 20,
                onClick = function()
                    set.favorite = not set.favorite
                    self:clear()
                    self.onRefresh()
                end
            }
        )
        favToggle:setImagePath("Icons/favorite_icon.png")
        self:buttonColoring(favToggle, set.favorite)
        local exportButton = osrs.Ui:imageButton(
            {
                sameLine = true,
                tooltip = "Export Setup",
                sizeMode = 1,
                width = 20,
                height = 20,
                onClick = function()
                    self.onExport(set)
                end
            }
        )
        exportButton:setImagePath("Icons/export_icon.png")
        local deleteButton = osrs.Ui:textButton(
            {
                sameLine = true,
                tooltip = "Delete",
                text = "x",
                onClick = function()
                    self.popupMenu:deletePopup("Set", set.name, true)
                end
            }
        )
        if self.compactMode then 
            viewButton.sameLine = true
            panel.height = 35
            UI = {setName, editButton, viewButton}
        else
            UI = {setName, editButton, viewButton, orderToggle, highlightToggle, favToggle, exportButton, deleteButton}
        end
        if self.sectionMode then
            if self.sectionMode and set.sections[1] == nil then
                self:addToWindow(panel, UI)
                self:addToWindow(self.sections[#self.sections], {panel})
            else
                for index, section in pairs(set.sections) do
                    for i, header in pairs(self.sections) do
                        if header.text == section then
                            local panel = self:clearPanelHelper()
                            local newEditButton = self:editButtonHelper(index, set, total)
                            UI[2] = newEditButton
                            self:addToWindow(panel, UI)
                            self:addToWindow(self.sections[i], {panel})
                        end
                    end
                end
            end
        else
            self:addToWindow(panel, UI)
            self:addToWindow(self.sets, {panel})
        end
    end,

    clearPanelHelper = function(self)
        local panel = osrs.Ui:pane(
            {
                sizeMode = 1,
                width = self.window.width - 20,
                height = 70,
                scrollable = false,
            }
        )
        return panel
    end,

    editButtonHelper = function(self, sectionIndex, set, total)
        local editButton = osrs.Ui:textButton(
            {
                sameLine = true,
                text = "Edit",
                onClick = function()
                    self.popupMenu:holderPopup(set, set.index, total, "Inventory Setup", 
                    sectionIndex, self.sections, self.sectionMode)
                end
            }
        )
        return editButton
    end,

    addToWindow = function(self, window, items)
        for index, item in pairs(items) do 
            window:addChild(item)
        end
    end,

    addCancelButton = function(self, window)
        cancelButton = osrs.Ui:textButton(
            {
                text = "Cancel",
                onClick = function()
                    osrs.Ui.canvas:removeChild(window)
                end
            }
        )
        window:addChild(cancelButton)
    end,

    -- condense this somehow
    searchItems = function(self, sets, sections)
        if self.sectionMode then
            for index, section in pairs(self.sections) do
                self.panel:removeChild(section)
            end
        elseif self.sets ~= nil then
            self.panel:removeChild(self.sets)
        end
        self.panel:removeChild(self.themeDropdown)
        self.panel:removeChild(self.closeButton)
        self.sets = osrs.Ui:grid(
            { 
                columns = 1,
                height = 75,
                width = self.window.width,
                sizeMode = 1,
                border = false,
            }
        )
        if #sets ~= 0 then 
            if self.compactMode then 
                self.sets.height = #sets * 40
            else
                self.sets.height = #sets * self.sets.height
            end
        end
        self.sections = {}
        if self.pastSearchValue ~= "" then
            result = self:searchSets(sets)
            sets = result["sets"]
            sections = result["sections"]
        end
        if self.sectionMode then
            self:setupSections(sections, sets)
        end
        self:setupSets(sets)
        if self.sectionMode then
            for index, section in pairs(self.sections) do
                self.panel:addChild(section)
            end
        else
            self.panel:addChild(self.sets)
        end
        self.panel:addChild(self.themeDropdown)
        self.panel:addChild(self.closeButton)
    end,

    show = function(self)
        self.panel:addChild(self.createButton)
        self.panel:addChild(self.sortingToggle)
        self.panel:addChild(self.compactToggle)
        self.panel:addChild(self.sectionToggle)
        self.panel:addChild(self.importButton)
        self.panel:addChild(self.searchBar)
        if self.sectionMode then
            for index, section in pairs(self.sections) do
                self.panel:addChild(section)
            end
        else
            self.panel:addChild(self.sets)
        end
        self.panel:addChild(self.themeDropdown)
        self.panel:addChild(self.closeButton)
        self.window:addChild(self.panel)
        ThemeTool:update(self.window)
        osrs.Ui.canvas:addChild(self.window)
    end,
    
    clear = function(self)
        self.panel:removeChild(self.createButton)
        self.panel:removeChild(self.sortingToggle)
        self.panel:removeChild(self.compactToggle)
        self.panel:removeChild(self.sectionToggle)
        self.panel:removeChild(self.importButton)
        self.panel:removeChild(self.searchBar)
        if self.sectionMode then
            for index, section in pairs(self.sections) do
                self.panel:removeChild(section)
            end
        else
            self.panel:removeChild(self.sets)
        end
        self.panel:removeChild(self.themeDropdown)
        self.panel:removeChild(self.closeButton)
        self.window:removeChild(self.panel)
        osrs.Ui.canvas:removeChild(self.window)
    end
}

return InventorySetupWindow